/**
* @file tuya_sdk_callback.c
* @brief Common process - adapter the flash api
* @version 0.1
* @date 2021-11-10
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/
#include <string.h>
#include "tkl_memory.h"

#include "tkl_endpoint_register.h"
#include "tal_log.h"
#include "tal_sleep.h"
#include "tal_system.h"
#include "tal_memory.h"
#include "tal_sw_timer.h"
#include "tal_zcl_scene.h"
#include "tal_network_mgr.h"
#include "tal_data_receive.h"
#include "tal_reset_factory.h"
#include "tal_heartbeat.h"
#include "tal_mf_test.h"
#include "tal_attribute_rw.h"

#include "app_common.h"
#include "app_dev_register.h"

#include "tuya_sdk_callback.h"

#if APP_KEY_SUPPORT
#include "app_key.h"
#endif
#if APP_LED_SUPPORT
#include "app_led.h"
#endif
#if APP_BATTERY_SUPPORT
#include "app_battery.h"
#endif
#if APP_GPIO_SUPPORT
#include "app_gpio.h"
#endif
#if IAS_ZONE_SUPPORT
#include "app_ias_zone.h"
#endif

//app event and handler
TIMER_ID etimer_preheat_event_id;
TIMER_ID etimer_network_up_delay_event_id;

STATIC VOID_T app_preheat_event_handler(TIMER_ID timer_id, VOID_T *arg);
STATIC VOID_T app_network_up_delay_event_handler(TIMER_ID timer_id, VOID_T *arg);

//app event create
STATIC VOID_T app_event_init(VOID_T);
//debug uart init
STATIC VOID_T app_debug_uart_init(VOID_T);
//debug output create
STATIC VOID_T app_uart_output(IN CONST  CHAR_T *str);
//oem init
STATIC VOID_T app_oem_init(VOID_T);

/**
 * @description: app preheat and network up delay event create
 * @param[in] none
 * @return: none
 */
STATIC VOID_T app_event_init(VOID_T)
{
    tal_sw_timer_create(app_preheat_event_handler, NULL, &etimer_preheat_event_id);
    tal_sw_timer_create(app_network_up_delay_event_handler, NULL, &etimer_network_up_delay_event_id);
}

/**
 * @description: preheat event handler
 * @param [in] timer_id: timer id
 * @param[in] args: null
 * @return: none
 */
STATIC VOID_T app_preheat_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
#if IAS_ZONE_SUPPORT
    app_zone_control_preheat_init();
#endif
#if APP_GPIO_SUPPORT
    app_gpio_preheat_init();
#endif
#if APP_LED_SUPPORT
    app_led_preheat_init();
#endif
}

/**
 * @description: network up delay event
 * @param [in] timer_id: timer id
 * @param[in] args: null
 * @return: none
 */
STATIC VOID_T app_network_up_delay_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    TAL_PR_TRACE("start report full\r\n");
}
/**
 * @brief [TUYA_WEAK_ATTRIBUTE function],set device version according to ep_id
 * 
 * @param[in] ep_id 
 * @return TKL_DEV_VER_T 
 */
TKL_DEV_VER_E tkl_zg_endpoint_to_device_version(UINT8_T ep_id){
    return 0xa0;
}
extern void tuya_cli_pre_init(void);
/**
 * @brief pre-sleep callback
 */
STATIC VOID_T app_pre_sleep_cb(VOID_T)
{
    TAL_PR_TRACE("app_pre_sleep_cb\r\n");
}

/**
 * @brief post-wakeup callback
 */
STATIC VOID_T app_post_wakeup_cb(VOID_T)
{
    TAL_PR_TRACE("app_post_wakeup_cb\r\n");
}
/**
 * @description: oem init
 * @param [in] none
 * @return: none
 */
STATIC VOID_T app_oem_init(VOID_T)
{
#if APP_BATTERY_SUPPORT
    STATIC UINT8_T ch = 0;
    APP_BATTERY_INFO_T bat_info = {
    .cap_first_delay_time = 2 * 60 * 1000,
    .cap_max_period_time =  3 * 60 * 1000,
    .adc.ch_nums = 1,
    .adc.ref_vol = 3300,
    .adc.width = 8,
    .adc.type = TUYA_ADC_INNER_SAMPLE_VOL,
    .adc.ch_list = 1<<ch};
    app_battery_config_init(&bat_info, NULL, 0);
#endif

#if APP_KEY_SUPPORT
    key_config_t key_info = APP_KEY_CONFIG_DEFAULT;
    app_key_oem_init(key_info);
#endif    

#if APP_LED_SUPPORT 
    APP_LED_INFO_T led_info = APP_LED_CONFIG_DEFAULT;
    app_led_oem_init(&led_info);
#endif

#if APP_GPIO_SUPPORT
    APP_GPIO_INFO_T gpio_info = APP_GPIO_SENSOR_CONFIG_DEFAULT;
    app_gpio_oem_init(&gpio_info);
#endif
}

/**
 * @brief Generally used for peripheral initialization
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_first(VOID_T)
{
    app_debug_uart_init();
    tal_log_create_manage_and_init(TAL_LOG_LEVEL_TRACE, 128, app_uart_output);

    tal_mf_test_disable_beacon_test();
    TAL_PR_DEBUG("/*********first init*********,reason=%d/\r\n",tal_system_get_reset_reason(NULL));
    return OPRT_OK;
}

/**
 * @brief Generally used for register zigbee device
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_second(VOID_T)
{
    dev_zigbee_init();

    UINT8_T attr = 0x02;
    OPERATE_RET ret = tal_zg_write_attribute(0x01,
                                            CLUSTER_IAS_ZONE_CLUSTER_ID,
                                            ATTR_ZONE_ID_ATTRIBUTE_ID,
                                            &attr,
                                            ATTR_INT8U_ATTRIBUTE_TYPE);
    TAL_PR_TRACE("attr[%d], status[%d]\r\n",attr, ret);

    attr = 0;
    ret = tal_zg_read_attribute(0x01,
                                   CLUSTER_IAS_ZONE_CLUSTER_ID,
                                   ATTR_ZONE_ID_ATTRIBUTE_ID,
                                   (VOID_T *)&attr,
                                   1);
    TAL_PR_TRACE("attr[%d], status[%d]\r\n",attr, ret);

    TAL_PR_TRACE("/*********1.0.0 second init*********/\r\n");
    return OPRT_OK;
}

/**
 * @brief Generally used for initialization before manufacturing test
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_third(VOID_T)
{
    TAL_PR_TRACE("/*********third init*********/\r\n");
    // app_oem_init();
    app_event_init();
    tal_sw_timer_start(etimer_preheat_event_id, 500, TAL_TIMER_ONCE);
    return OPRT_OK;
}

/**
 * @brief Generally used for initialization after manufacturing test
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_last(VOID_T)
{
    app_oem_init();
    TUYA_SLEEP_CB_T v_cb = {app_pre_sleep_cb, app_post_wakeup_cb};
    tal_cpu_sleep_callback_register(&v_cb);
    tal_uart_deinit(USER_UART0);
    app_debug_uart_init();

    TAL_PR_TRACE("/*********last init*********/\r\n");
    return OPRT_OK;
}

/**
 * @brief user-defined callback interface in main loop.do not block!!!
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_main_loop(VOID_T)
{
    return OPRT_OK;
}

VOID_T tal_zg_ota_start_callback(UINT_T image_size, VOID_T* argv)
{
    tuya_zg_poll_interval_set(200);
    tal_zg_poll_start();
}

VOID_T tal_zg_ota_end_callback(UINT8_T result)
{
    tuya_zg_poll_interval_set(1000);
    tal_zg_poll_stop();
}

/**
 * @brief general message receive callback
 * 
 * @param msg 
 * @return TAL_MSG_RET_E 
 */
TAL_MSG_RET_E tal_zcl_general_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{
    //TAL_PR_TRACE("app gen msg cb: cluster 0x%02x, cmd %d\r\n", msg->cluster, msg->command);

    return ZCL_MSG_RET_SUCCESS;
}

/**
 * @brief post-write attribute callback
 *
 * @param[in] ep_id:      endpoint id
 * @param[in] cluster_id: cluster id
 * @param[in] attr_rec:   point to attribute record 
 * @return write status
 */
TAL_WRITE_RET_E tal_zg_post_write_attribute_callback(UINT8_T ep_id, 
                                                    UINT16_T cluster_id,
                                                    TAL_ATTR_REC_T *attr_rec)
{
    if (IAS_ZONE_ENDPOINT == ep_id) {
        switch (cluster_id) {
            case CLUSTER_IDENTIFY_CLUSTER_ID: {
                if (ATTR_IDENTIFY_TIME_ATTRIBUTE_ID == attr_rec->attr_id) {
                }
                break;
            }
#if IAS_ZONE_SUPPORT
            case CLUSTER_IAS_ZONE_CLUSTER_ID: {
                if( ATTR_IAS_CIE_ADDRESS_ATTRIBUTE_ID == attr_rec->attr_id ) {
                    ias_zone_cie_address_write_callback();
                }
                break;
            }
#endif
            default: {
                break;
            }
        }
    }
    return WRITE_RET_SUCCESS;
}

/**
 * @brief specific message receive callback
 * 
 * @param msg 
 * @return TAL_MSG_RET_E 
 */
TAL_MSG_RET_E tal_zcl_specific_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{
    TAL_PR_TRACE("app spec msg cb: cluster 0x%02x, cmd 0x%02x\r\n", msg->cluster, msg->command);

    ZIGBEE_CMD_E app_cmd_type = ZIGBEE_CMD_SINGLE;
    if (msg->mode == ZG_UNICAST_MODE)
    {
        app_cmd_type = ZIGBEE_CMD_SINGLE;
        TAL_PR_TRACE("receive single message");
    }
    else
    {
        app_cmd_type = ZIGBEE_CMD_GROUP;
        TAL_PR_TRACE("receive group message");
    }
    TAL_PR_TRACE("app command type %d", app_cmd_type);
    switch (msg->cluster)
    {
#if IAS_ZONE_SUPPORT
        case CLUSTER_IAS_ZONE_CLUSTER_ID: {
            ias_zone_cluster_server_command_handler(msg->command, msg->payload, msg->length, app_cmd_type);
            break;
        }
#endif
        default:
            break;
    }

    return ZCL_MSG_RET_SUCCESS;
}

/**
 * @brief pre-save scene callback
 * 
 * @param ep_id 
 * @param scene_id 
 * @param group_id 
 * @return VOID_T 
 */
VOID_T tal_zg_scene_pre_save_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id)
{
    //TAL_PR_TRACE("scene pre save: ep %d, sce %d, gp 0x%02x\r\n", ep_id, scene_id, group_id);
}

/**
 * @brief save scene callback(add scene and store scene)
 * 
 * @param ep_id 
 * @param scene_id 
 * @param group_id 
 * @param data 
 * @return VOID_T 
 */
VOID_T tal_zg_scene_save_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id, TAL_SCENE_DATA_T *data)
{
    //TAL_PR_TRACE("scene save: ep %d, sce %d, gp 0x%02x, type %d\r\n", ep_id, scene_id, group_id, data->type);
}
/**
 * @brief recall scene callback
 *
 * @param[in]   ep_id:     endpoint id
 * @param[in]   scene_id:  scene id
 * @param[in]   group_id:  group id
 * @param[in]   time100ms: scene transition time(bat:100ms)
 * @param[in]   data:      point to scene data
 * @return  VOID_T
 */
VOID_T tal_zg_scene_recall_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id, UINT_T time100ms, TAL_SCENE_DATA_T *data)
{
    //TAL_PR_TRACE("scene recall: ep %d, sce %d, gp 0x%02x, type %d\r\n", ep_id, scene_id, group_id, data->type);
}

/**
 * @brief add group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_add_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_TRACE("ep_id=%d, group_id=0x%04x", ep_id, group_id);
}

/**
 * @brief view group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_view_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_TRACE("ep_id=%d, group_id=0x%04x", ep_id, group_id);
}

/**
 * @brief remove group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_remove_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_TRACE("ep_id=%d, group_id=0x%04x", ep_id, group_id);
}

/**
 * @brief zigbee network network change callback(user can rewrite this API)
 * 
 * @param[in]   status: network status
 * @return VOID_T
 */
VOID_T tal_zg_nwk_status_changed_callback(TAL_ZG_NWK_STATUS_E status)
{
    switch (status)
    {
        case TAL_ZG_NWK_POWER_ON_LEAVE:
        {
            TAL_PR_TRACE("power_on_leave---\r\n");
            break;
        }
        case TAL_ZG_NWK_POWER_ON_ONLINE:
        {
            TAL_PR_TRACE("power_on_online---\r\n");
            tal_sw_timer_start(etimer_network_up_delay_event_id, 5000, TAL_TIMER_ONCE);
            break;
        }
        case TAL_ZG_NWK_JOIN_START:
        {
            TAL_PR_TRACE("nwk_join_start---\r\n");
#if APP_LED_SUPPORT
            dev_start_join_indicate();
#endif
            break;
        }
        case TAL_ZG_NWK_JOIN_OK:
        {
            TAL_PR_TRACE("nwk_join_ok---\r\n");
#if APP_LED_SUPPORT
            dev_join_success_indicate();
#endif
            tal_sw_timer_start(etimer_network_up_delay_event_id, 5000, TAL_TIMER_ONCE);
            app_battery_start();
            break;
        }
        case TAL_ZG_NWK_REJOIN_OK:
        {
            TAL_PR_TRACE("nwk_rejoin_ok---\r\n");
            app_battery_start();
            break;
        }
        case TAL_ZG_NWK_JOIN_TIMEOUT:
        {
            TAL_PR_TRACE("nwk_join_timeout---\r\n");
            break;
        }
        case TAL_ZG_NWK_LOST:
        {
            TAL_PR_TRACE("nwk_lost---\r\n");
            break;
        }
        case TAL_ZG_NWK_REMOTE_LEAVE:
        {
            TAL_PR_TRACE("nwk_remote_leave---\r\n");
            tal_zg_join_start(ZIGBEE_JOIN_TIMEOUT_MS);
            TAL_PR_TRACE("zigbee join start %d.\r\n", ZIGBEE_JOIN_TIMEOUT_MS);
            break;
        }
        case TAL_ZG_NWK_LOCAL_LEAVE:
        {
            TAL_PR_TRACE("nwk_local_leave---\r\n");
            break;
        }
        case TAL_ZG_NWK_MF_TEST_LEAVE:
        {
            TAL_PR_TRACE("nwk_mf_test_leave---\r\n");
            break;
        }
        default:
        {
            break;
        }
    }
}
/**
 * @brief reset factory default callback
 *
 * @param[in]   type: reset factory type
 * @return  VOID_T
 */
VOID_T tal_zg_reset_factory_default_callback(TAL_RESET_TYPE_T type)
{
#if IAS_ZONE_SUPPORT
    ias_zone_unenroll();
#endif
    TAL_PR_TRACE("receive reset to factory default cmd %d\r\n", type);
}

/**
 * @note uart init 
 * @param [in] none 
 * @return: OPERATE_RET
 */
STATIC VOID_T app_debug_uart_init(VOID_T)
{
    TAL_UART_CFG_T cfg;
    tal_system_memset(&cfg, 0, SIZEOF(TAL_UART_CFG_T));

    cfg.rx_buffer_size = 128;
    cfg.open_mode = 0;
    cfg.base_cfg.baudrate = 115200;
    cfg.base_cfg.parity = TUYA_UART_PARITY_TYPE_NONE;
    cfg.base_cfg.databits = TUYA_UART_DATA_LEN_8BIT;
    cfg.base_cfg.stopbits =TUYA_UART_STOP_LEN_1BIT;
    cfg.base_cfg.flowctrl = TUYA_UART_FLOWCTRL_NONE;
    tal_uart_init(USER_UART0, &cfg);
}

/**
 * @brief 
 * 
 * @param str 
 * @return VOID_T 
 */
STATIC VOID_T app_uart_output(IN CONST  CHAR_T *str)
{
    tal_uart_write(USER_UART0, str, strlen(str));
}
VOID_T tal_beacon_mf_test_callback(VOID_T)
{
    TAL_PR_TRACE("enter beacon test\r\n");
}

VOID_T tal_mf_test_pre_start_callback(VOID_T *args)
{
    TAL_PR_TRACE("enter dongle-beacon start\r\n");
}

VOID_T tal_mf_test_end_callback(UINT8_T *data)
{
    TAL_PR_TRACE("enter dongle-beacon end, value %d\r\n", *data);
}

